#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <omp.h>

// Do not edit this function
void input_generator(int n, int dim, double **M) {
    int i, j;
    for (i = 0; i < n; i++) {
        M[i][0] = 0;
        for (j = 0; j < dim - 1; j++) {
            M[i][j + 1] = ((i + 1) / (double)n) * (j + 1);
        }
    }
}

int main(int argc, char ** argv) {
    int i, j, k;
    int points = 5, n = 3;
    double sum, sum_avg, mul, x = 2.5;
    double *X, **M;
    double start_time = 0, end_time = 0;

    if(argc == 1) {
      omp_set_num_threads(1);
    } else if(argc == 2) {
      omp_set_num_threads(atoi(argv[1]));
    } else {
      printf("Error: Expected number of args either 0 or 1\n");
      exit(EXIT_FAILURE);
    }

    printf("points = %d, n = %d\n", points, n);
    printf("x = %f\n", x);

    M = (double **)malloc(n * sizeof(double *));
    for (i = 0; i < n; i++) {
        M[i] = (double *)malloc(points * sizeof(double));
    }
    X = (double *)malloc(points * sizeof(double));
    for (i = 0; i < points; i++) {
        X[i] = i;
    }
    input_generator(n, points, M);

    // You have to parallelize these three loops. Do not edit any other loops in the code.
    // Place "start_time" here
    sum_avg = 0;
    for (i = 0; i < n; i++) {
        sum = 0;
        for (j = 0; j < points; j++) {
            mul = 1;
            for (k = 0; k < points; k++) {
                if (j == k || fabs(X[j] - X[k]) < 1e-9) continue;  // Prevent division by zero
                mul *= ((x - X[k]) / (X[j] - X[k]));
            }
            sum += mul * M[i][j];
        }
        sum_avg += sum;
    }
    // Place "end_time" here

    printf("sum_avg(%f) = %f\n", x, sum_avg / (double)n);

    printf("Elapsed time: %.10f sec\n", (end_time - start_time));

    for (i = 0; i < n; i++) {
        free(M[i]);
    }
    free(M);
    free(X);
    
    return 0;
}
